@@ -54,6 +54,11 @@
 #' @return A vector with the names of the created output files
 #'   (just created or already existing).
 #' @author Luigi Ranghetti, phD (2019) \email{luigi@@ranghetti.info}
+#' @references L. Ranghetti, M. Boschetti, F. Nutini, L. Busetto (2020).
+#'  "sen2r": An R toolbox for automatically downloading and preprocessing 
+#'  Sentinel-2 satellite data. _Computers & Geosciences_, 139, 104473. DOI: 
+#'  \href{https://doi.org/10.1016/j.cageo.2020.104473}{10.1016/j.cageo.2020.104473}, 
+#'  URL: \url{http://sen2r.ranghetti.info/}.
 #' @note License: GPL 3.0
 #' @importFrom jsonlite fromJSON
 #' @export
@@ -139,8 +144,6 @@ s2_translate <- function(infile,
       "\u00A0\u00A0gdalinfo(formats=TRUE)[grep(\"yourformat\", gdalinfo(formats=TRUE))]")
   }
   
-  # Check GDAL installation
-  check_gdal(abort=TRUE)
   # Retrieve xml required metadata
   infile_meta <- safe_getMetadata(
     infile, 
